{% block flag_icon %}
<svg class="flag-icon {% if has_flagged %}user-has-flagged{% else %}user-has-not-flagged{% endif %} {% block cls_flag_icon_img %}{% endblock cls_flag_icon_img %}" xmlns="http://www.w3.org/2000/svg" width="18" height="18" viewBox="0 0 24 24" stroke="#427297" stroke-width="1.7" stroke-linecap="round" stroke-linejoin="round">
    <path d="M4 15s1-1 4-1 5 2 8 2 4-1 4-1V3s-1 1-4 1-5-2-8-2-4 1-4 1z"></path><line x1="4" y1="22" x2="4" y2="15"></line>
</svg>
{% endblock flag_icon %}
